# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-04-13 05:31
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('fineuploader', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='temporary',
            name='file_obj',
        ),
        migrations.RemoveField(
            model_name='temporary',
            name='original_filename',
        ),
        migrations.RemoveField(
            model_name='temporary',
            name='uuid',
        ),
    ]



# -*- coding: utf-8 -*-

from datetime import timedelta

from django.db import models
from django.utils import timezone
from django.contrib.contenttypes.fields import GenericRelation

from django.utils.translation import ugettext_lazy as _

from attachments.models import Attachment

from .managers import TemporaryManager
from .conf import settings


class Temporary(models.Model):

    formid = models.CharField(max_length=128)
    field_name = models.CharField(max_length=256, null=True, blank=True)

    attachments = GenericRelation(Attachment)

    # for internal use...

    timestamp = models.DateTimeField(default=timezone.now)

    objects = TemporaryManager()

    class Meta(object):
        verbose_name = _('temporary upload')
        verbose_name_plural = _('temporary uploads')
        # Query string to get back existing uploaded file is using form_id and field_name
        index_together = (
            ('formid', 'field_name'),
        )
        ordering = ['-timestamp']

    @property
    def is_expired(self):
        EXPIRY_AGE = settings.FINEUPLOADER_TEMPORARY_AGE

        if (self.created + timedelta(seconds=EXPIRY_AGE)) <= timezone.localtime(timezone.now()):
            return True
        else:
            return False



# -*- coding: utf-8 -*-

import json

from django.forms import ClearableFileInput
from django.contrib.contenttypes.models import ContentType

from attachments.models import Attachment


class FineInput(ClearableFileInput):
    template_name = 'fineuploader/fineinput.html'
    target_object = None

    def __init__(self, file_limit=4, size_limit=10485760, attrs=None):
        self.file_limit, self.size_limit = file_limit, size_limit
        super(FineInput, self).__init__(attrs)

    def render(self, name, value, attrs=None, renderer=None):
        target_object = self.target_object

        context = self.get_context(name, value, attrs)
        context['widget'].update({
            'options': {
                'fileLimit': self.file_limit,
                'sizeLimit': self.size_limit,
            },
            'content_type': ContentType.objects.get_for_model(target_object),
            'object': target_object,
        })
        return self._render(self.template_name, context, renderer)

    def value_from_datadict(self, data, files, name):
        return Attachment.objects.attachments_for_object(self.target_object) or None



# -*- coding: utf-8 -*-

import uuid

from django import forms
from django.contrib.contenttypes.models import ContentType

from attachments.models import Attachment

from .models import Temporary
from .formfields import FineFileField


class FineFormMixin(object):
    formid_field_name = 'formid'
    request = None

    def __init__(self, *args, **kwargs):
        if kwargs.get('initial', None) is None:
            kwargs['initial'] = {}
        if not self.formid_field_name in kwargs['initial']:
            kwargs['initial'].update({
                self.formid_field_name: str(uuid.uuid4())})
        super(FineFormMixin, self).__init__(*args, **kwargs)

        self.fields[self.formid_field_name] = forms.CharField(
            widget=forms.HiddenInput, initial=kwargs['initial'][self.formid_field_name], required=False)

        formid = self.data.get(self.formid_field_name, self.initial.get(self.formid_field_name))
        for f in self.fields:
            if not isinstance(self.fields[f], FineFileField):
                continue
            
            self.fields[f].widget.target_object = self.get_target_object(formid, f)

    def get_target_object(self, formid=None, field_name=None):
        if hasattr(self, 'instance') is True:
            if getattr(self.instance, 'pk', False):
                return self.instance

        # instance not found, trying to get temporary object...
        if formid and field_name:
            t, created = Temporary.objects.get_or_create(
                formid=formid, field_name=field_name)

            return t

        raise NotImplementedError

    def handle_uploads(self, target_object, *args, **kwargs):
        for f in self.fields:
            if not isinstance(self.fields[f], FineFileField):
                continue

            for a in self.cleaned_data[f]:
                self.save_attachment(target_object, a, f, *args, **kwargs)

    def save_attachment(self, target_object, attachment, field_name, *args, **kwargs):
        attachment.content_type_id = ContentType.objects.get_for_model(
            target_object).pk
        attachment.object_id = target_object.pk
        attachment.save()

        return attachment



# -*- coding: utf-8 -*-

import logging

from django.utils import timezone
from django.core.files.base import ContentFile
from django.core.urlresolvers import get_callable
from django.core.exceptions import PermissionDenied
from django.contrib.contenttypes.models import ContentType

from attachments.models import Attachment

from .ajaxuploader.backends import local as backend
from .models import Temporary
from .conf import settings

logger = logging.getLogger(__name__)


def get_target_object(request, context):
    obj = ContentType.objects.get_for_id(
        context['content_type']).get_object_for_this_type(pk=context['object_id'])

    if isinstance(obj, Temporary):
        return obj

    # TODO: check for permissions ?

    return obj


class LocalUploadBackend(backend.LocalUploadBackend):
    FILENAME_FUNCTION = getattr(
        settings, 'FINEUPLOADER_FILENAME_FUNCTION', None)

    def setup(self, request, filename, *args, **kwargs):
        super(LocalUploadBackend, self).setup(
            request, filename, *args, **kwargs)
        self.request = request  # save request for a later use

    def upload_complete(self, request, filename, *args, **kwargs):
        try:
            response = super(LocalUploadBackend, self).upload_complete(
                request, filename, *args, **kwargs)
        except Exception, e:
            return self.failure(unicode(e), request)

        return response

    def failure(self, msg, request):
        self.log_exception(msg, request)
        return {'success': False, 'error': msg, 'preventRetry': True}

    def update_filename(self, request, filename, *args, **kwargs):
        # use custom photo filename function if any
        func = self.FILENAME_FUNCTION
        if func is None:
            func = lambda x: x

        # find callable by string
        if isinstance(func, str):
            func = get_callable(func)

        value = super(LocalUploadBackend, self).update_filename(
            request, func(filename), *args, **kwargs)
        return value

    def upload(self, uploaded, filename, raw_data, *args, **kwargs):
        success = super(LocalUploadBackend, self).upload(
            uploaded, filename, raw_data, *args, **kwargs)
        if success is False:
            self.log_exception("Failed to upload: %s" % filename, self.request)
        return success

    def log_exception(self, msg, request):
        logger.error(msg, exc_info=True, extra={'stack': True})


class FineUploadBackend(LocalUploadBackend):

    def upload_complete(self, request, filename, *args, **kwargs):
        response = super(FineUploadBackend, self).upload_complete(
            request, filename, *args, **kwargs)

        if request.POST.get('qqfilename'):
            original_filename = request.POST['qqfilename']
        else:
            original_filename = request.FILES['qqfile'].name

        try:
            target_object = self.get_target_object(request, request.POST)

            model_info = {
                'creator': request.user,
                'content_type': ContentType.objects.get_for_model(target_object.__class__),
                'object_id': target_object.pk,
            }

            a = Attachment(attachment_file=None, **model_info)

            with open(self._path) as fh:
                a.attachment_file.save(original_filename, ContentFile(fh.read()), save=True)
            a.save()

            response.update({'newUuid': a.pk})

        except Exception, e:
            return self.failure(unicode(e), request)

        response.update({
            'content_type': request.POST['content_type'],
            'object_id': request.POST['object_id'],
        })

        return response

    def get_target_object(self, request, context):
        return get_target_object(request, context)



# -*- coding: utf-8 -*-

try:
    import json
except ImportError:
    from django.utils import simplejson as json
from django.core.serializers.json import DjangoJSONEncoder
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse, Http404, HttpResponseNotAllowed

from attachments.models import Attachment

from .ajaxuploader.views import AjaxFileUploader
from .backends import FineUploadBackend, get_target_object

request_endpoint = AjaxFileUploader(backend=FineUploadBackend)


# TODO: replace with class-based view
def session_endpoint(request, *args, **kwargs):
    if request.method == "GET":
        try:
            target_object = get_target_object(request, request.GET)
        except Exception, e:
            return HttpResponse(json.dumps(unicode(e), cls=DjangoJSONEncoder), content_type="text/html; charset=utf-8", status=400)

        response = []
        for a in Attachment.objects.attachments_for_object(target_object):
            response.append({
                'name': a.filename,
                'uuid': a.pk,
                'size': a.attachment_file.size,
            })

        # although "application/json" is the correct content type, IE throws a fit
        return HttpResponse(json.dumps(response, cls=DjangoJSONEncoder), content_type="text/html; charset=utf-8")
    else:
        return HttpResponseNotAllowed(['GET'])


# TODO: replace with class-based view
def delete_endpoint(request, *args, **kwargs):
    if request.method == "POST":
        try:
            target_object = get_target_object(request, request.POST)
        except Exception, e:
            return HttpResponse(json.dumps({unicode(e)}, cls=DjangoJSONEncoder), content_type="text/html; charset=utf-8", status=400)

        try:
            Attachment.objects.attachments_for_object(target_object).get(
                pk=request.POST['qquuid']).delete()
        except ObjectDoesNotExist, e:
            raise Http404

        # although "application/json" is the correct content type, IE throws a fit
        return HttpResponse(json.dumps({'success': True}, cls=DjangoJSONEncoder), content_type="text/html; charset=utf-8")
    else:
        return HttpResponseNotAllowed(['POST'])




# -*- coding: utf-8 -*-

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from attachments.admin import AttachmentInlines
from attachments.models import Attachment

from .models import Temporary


@admin.register(Temporary)
class TemporaryAdmin(admin.ModelAdmin):
    list_display = ['formid', 'count', 'timestamp']
    readonly_fields = ['formid', 'field_name', 'timestamp']
    date_hierarchy = 'timestamp'
    inlines = [AttachmentInlines]

    def count(self, obj):
        return Attachment.objects.attachments_for_object(obj).count()
    count.short_description = _("no. of attachments")
